]> git.sesse.net Git - nageru/blobdiff - mixer.h
Remove the Mesa patch; replace with a recommendation to run 11.2.x.
[nageru] / mixer.h
diff --git a/mixer.h b/mixer.h
index af976985c91b3a52e442debb6e19e1c89c0e41ca..6a58575febd61d230b5e0cb7c65a6771a66248e5 100644 (file)
--- a/mixer.h
+++ b/mixer.h
@@ -24,7 +24,7 @@
 #include "bmusb/bmusb.h"
 #include "alsa_output.h"
 #include "ebu_r128_proc.h"
-#include "h264encode.h"
+#include "video_encoder.h"
 #include "httpd.h"
 #include "pbo_frame_allocator.h"
 #include "ref_counted_frame.h"
@@ -37,7 +37,7 @@
 #include "input_state.h"
 #include "correlation_measurer.h"
 
-class H264Encoder;
+class QuickSyncEncoder;
 class QSurface;
 namespace movit {
 class Effect;
@@ -71,10 +71,12 @@ class QSurfaceFormat;
 //
 // N is reduced as follows: If the queue has had at least one spare frame for
 // at least 50 (master) frames (ie., it's been too conservative for a second),
-// we reduce N by 1 and reset the timers.
+// we reduce N by 1 and reset the timers. TODO: Only do this if N ever actually
+// touched the limit.
 //
// Whenever the queue is starved (we needed a frame but there was none), N was
// obviously too low, so we increment N. We will never set N above 5, though.
 class QueueLengthPolicy {
 public:
        QueueLengthPolicy() {}
@@ -82,6 +84,7 @@ public:
                this->card_index = card_index;
                safe_queue_length = 0;
                frames_with_at_least_one = 0;
+               been_at_safe_point_since_last_starvation = false;
        }
 
        void update_policy(int queue_length);  // Give in -1 for starvation.
@@ -91,6 +94,7 @@ private:
        unsigned card_index;  // For debugging only.
        unsigned safe_queue_length = 0;  // Called N in the comments.
        unsigned frames_with_at_least_one = 0;
+       bool been_at_safe_point_since_last_starvation = false;
 };
 
 class Mixer {
@@ -143,6 +147,25 @@ public:
                output_channel[output].set_frame_ready_callback(callback);
        }
 
+       // TODO: Should this really be per-channel? Shouldn't it just be called for e.g. the live output?
+       typedef std::function<void(const std::vector<std::string> &)> transition_names_updated_callback_t;
+       void set_transition_names_updated_callback(Output output, transition_names_updated_callback_t callback)
+       {
+               output_channel[output].set_transition_names_updated_callback(callback);
+       }
+
+       typedef std::function<void(const std::string &)> name_updated_callback_t;
+       void set_name_updated_callback(Output output, name_updated_callback_t callback)
+       {
+               output_channel[output].set_name_updated_callback(callback);
+       }
+
+       typedef std::function<void(const std::string &)> color_updated_callback_t;
+       void set_color_updated_callback(Output output, color_updated_callback_t callback)
+       {
+               output_channel[output].set_color_updated_callback(callback);
+       }
+
        typedef std::function<void(float level_lufs, float peak_db,
                                   float global_level_lufs, float range_low_lufs, float range_high_lufs,
                                   float gain_staging_db, float final_makeup_gain_db,
@@ -167,6 +190,11 @@ public:
                return theme->get_channel_name(channel);
        }
 
+       std::string get_channel_color(unsigned channel) const
+       {
+               return theme->get_channel_color(channel);
+       }
+
        int get_channel_signal(unsigned channel) const
        {
                return theme->get_channel_signal(channel);
@@ -347,7 +375,7 @@ private:
        void place_rectangle(movit::Effect *resample_effect, movit::Effect *padding_effect, float x0, float y0, float x1, float y1);
        void thread_func();
        void schedule_audio_resampling_tasks(unsigned dropped_frames, int num_samples_per_frame, int length_per_frame);
-       void render_one_frame();
+       void render_one_frame(int64_t duration);
        void send_audio_level_callback();
        void audio_thread_func();
        void process_audio_one_frame(int64_t frame_pts_int, int num_samples);
@@ -367,7 +395,7 @@ private:
        GLuint cbcr_program_num;  // Owned by <resource_pool>.
        GLuint cbcr_vbo;  // Holds position and texcoord data.
        GLuint cbcr_position_attribute_index, cbcr_texcoord_attribute_index;
-       std::unique_ptr<H264Encoder> h264_encoder;
+       std::unique_ptr<VideoEncoder> video_encoder;
 
        // Effects part of <display_chain>. Owned by <display_chain>.
        movit::FlatInput *display_input;
@@ -388,7 +416,7 @@ private:
                        int64_t length;  // In TIMEBASE units.
                        bool interlaced;
                        unsigned field;  // Which field (0 or 1) of the frame to use. Always 0 for progressive.
-                       RefCountedGLsync ready_fence;  // Whether frame is ready for rendering.
+                       std::function<void()> upload_func;  // Needs to be called to actually upload the texture to OpenGL.
                        unsigned dropped_frames = 0;  // Number of dropped frames before this one.
                };
                std::queue<NewFrame> new_frames;
@@ -417,16 +445,25 @@ private:
                void output_frame(DisplayFrame frame);
                bool get_display_frame(DisplayFrame *frame);
                void set_frame_ready_callback(new_frame_ready_callback_t callback);
+               void set_transition_names_updated_callback(transition_names_updated_callback_t callback);
+               void set_name_updated_callback(name_updated_callback_t callback);
+               void set_color_updated_callback(color_updated_callback_t callback);
 
        private:
                friend class Mixer;
 
+               unsigned channel;
                Mixer *parent = nullptr;  // Not owned.
                std::mutex frame_mutex;
                DisplayFrame current_frame, ready_frame;  // protected by <frame_mutex>
                bool has_current_frame = false, has_ready_frame = false;  // protected by <frame_mutex>
                new_frame_ready_callback_t new_frame_ready_callback;
-               bool has_new_frame_ready_callback = false;
+               transition_names_updated_callback_t transition_names_updated_callback;
+               name_updated_callback_t name_updated_callback;
+               color_updated_callback_t color_updated_callback;
+
+               std::vector<std::string> last_transition_names;
+               std::string last_name, last_color;
        };
        OutputChannel output_channel[NUM_OUTPUTS];
@@ -71,10 +71,12 @@ class QSurfaceFormat;
 //
-// Whenever the queue is starved (we needed a frame but there was none), N was
-// obviously too low, so we increment N. We will never set N above 5, though.
+// Whenever the queue is starved (we needed a frame but there was none),
+// and we've been at N since the last starvation, N was obviously too low,
+// so we increment it. We will never set N above 5, though.
 class QueueLengthPolicy {
 public: